USE tempdb;
GO

IF OBJECT_ID(N'dbo.SomeDummyTable', N'U') IS NOT NULL
	DROP TABLE dbo.SomeDummyTable
GO

IF OBJECT_ID(N'dbo.ProcedureWithDML', N'P') IS NOT NULL
	DROP PROCEDURE dbo.ProcedureWithDML
GO

IF OBJECT_ID(N'dbo.ProcedureDynamicSql', N'P') IS NOT NULL
	DROP PROCEDURE dbo.ProcedureDynamicSql
GO

IF OBJECT_ID(N'dbo.ProcedureTempTable', N'P') IS NOT NULL
	DROP PROCEDURE dbo.ProcedureTempTable
GO

IF OBJECT_ID(N'dbo.ProcedureMultipleResultSets', N'P') IS NOT NULL
	DROP PROCEDURE dbo.ProcedureMultipleResultSets
GO

CREATE TABLE dbo.SomeDummyTable( dummy INTEGER );
GO

CREATE PROCEDURE dbo.ProcedureWithDML
	@StartIndex	AS INTEGER,
	@BatchSize AS SMALLINT
AS
BEGIN
	--SET NOCOUNT ON;	Commented deliberately to produce errors
	DELETE dbo.SomeDummyTable
	WHERE 1 = 0;

	SELECT	CAST(number AS SMALLINT) AS number,
			CAST(high - 1 AS TINYINT) AS high,
			CAST(low - 1 AS TINYINT) AS low
	FROM	master.dbo.spt_values
	WHERE	[type] = 'P'
			AND number >= @StartIndex
			AND number < @StartIndex + @BatchSize;
END
GO

CREATE PROCEDURE dbo.ProcedureDynamicSql
	@StartIndex	AS INTEGER,
	@BatchSize AS SMALLINT
AS
BEGIN
	--SET NOCOUNT ON;	Commented deliberately to produce errors
	DECLARE @SqlCmmd AS NVARCHAR(MAX) = N'
	SELECT	CAST(number AS SMALLINT) AS number,
			CAST(high - 1 AS TINYINT) AS high,
			CAST(low - 1 AS TINYINT) AS low
	FROM	master.dbo.spt_values
	WHERE	[type] = ''P''
			AND number >= @StartIndex
			AND number < @StartIndex + @BatchSize;';

	DECLARE @Params AS NVARCHAR(500) = N'@StartIndex AS INTEGER, @BatchSize AS SMALLINT';

	EXECUTE sp_executesql
		@statement	= @SqlCmmd,
		@params		= @Params,
		@StartIndex	= @StartIndex,
		@BatchSize	= @BatchSize;
END
GO

CREATE PROCEDURE dbo.ProcedureTempTable
	@StartIndex	AS INTEGER,
	@BatchSize AS SMALLINT
AS
BEGIN
	SELECT	CAST(number AS SMALLINT) AS number,
			CAST(high - 1 AS TINYINT) AS high,
			CAST(low - 1 AS TINYINT) AS low
	INTO	#Result
	FROM	master.dbo.spt_values
	WHERE	[type] = 'P'
			AND number >= @StartIndex
			AND number < @StartIndex + @BatchSize;

	SELECT	*
	FROM	#Result;
END
GO

CREATE PROCEDURE dbo.ProcedureMultipleResultSets
	@StartIndex	AS INTEGER,
	@BatchSize	AS SMALLINT,
	@Type		AS TINYINT = 0
AS
BEGIN
	IF @Type = 0
	BEGIN
		SELECT	CAST(number AS SMALLINT) AS number,
				CAST(high - 1 AS TINYINT) AS high,
				CAST(low - 1 AS TINYINT) AS low
		FROM	master.dbo.spt_values
		WHERE	[type] = 'P'
				AND number >= @StartIndex
				AND number < @StartIndex + @BatchSize;
	END
	ELSE
	BEGIN
		--New columns and different order.
		SELECT	CAST(high - 1 AS TINYINT) AS high,
				CAST(low - 1 AS TINYINT) AS low,	--> Just for fun change to SMALLINT and see what happens
				CAST(number AS SMALLINT) AS number,
				CAST(SYSUTCDATETIME() AS DATETIME2(0)) AS [timestamp],
				NEWID() AS [newid]
		FROM	master.dbo.spt_values
		WHERE	[type] = 'P'
				AND number >= @StartIndex
				AND number < @StartIndex + @BatchSize;		
	END;
END;
GO
